﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Media;
using XNA4;

namespace XNA4Sound
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont font;

        // Set the position of the model in world space, and set the rotation.
        Vector3 modelPosition = Vector3.Zero;
        float modelRotation = 0.0f;
        float modelRotationPitch = 0.0f;
        // Set the velocity of the model, applied each frame to the model's position.
        Vector3 modelVelocity = Vector3.Zero;
        // Set the position of the camera in world space, for our view matrix.
        Vector3 cameraPosition = new Vector3(0.0f, 50.0f, 5000.0f);

        /// <summary>
        /// input
        /// </summary>
        static InputHandler inputs;
        /// <summary>
        /// FPS Counter
        /// </summary>
        static FpsCounter fpsCounter;

        List<SoundNode<SoundEffect>> sounds = new List<SoundNode<SoundEffect>>();
        List<SoundNode<Song>> songs = new List<SoundNode<Song>>();

        static SoundNodeHandler soundHandler;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }


        protected override void Initialize()
        {
            /// <summary>
            /// inputs component
            /// </summary>
            inputs = new InputHandler(this);
            Components.Add(inputs);
            /// <summary>
            /// fps component
            /// </summary>
            fpsCounter = new FpsCounter(this);
            Components.Add(fpsCounter);
            /// <summary>
            /// soundNodeHandler component
            /// </summary>
            soundHandler = new SoundNodeHandler(this);
            Components.Add(soundHandler);

            base.Initialize();
        }

        // Set the 3D model to draw.
        Model myModel;

        // The aspect ratio determines how to scale 3d to 2d projection.
        float aspectRatio;

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            font = Content.Load<SpriteFont>("Arial"); 

            myModel = Content.Load<Model>("Model\\p1_wedge");

            sounds.Add(new SoundNode<SoundEffect>(this, "Sound\\engine_2", 1));
            //sounds.Add(new SoundNode<SoundEffect>(this, "Sound\\kaboom", 2));
            sounds.Add(new SoundNode<SoundEffect>(this, "Sound\\hyperspace_activate", 3));
            songs.Add(new SoundNode<Song>(this, "Sound\\Love is", 1));
            songs.Add(new SoundNode<Song>(this, "Sound\\회상", 2));
            songs[0].Play(false, 0.2f);

            soundHandler.Add("Sound\\아빠와 슈만(대사와 연주)", 1);
            soundHandler.Add("Sound\\Love is", 2);
            soundHandler.Add("Sound\\회상", 3);
            soundHandler.Add("Sound\\engine_2.wav", 4);
            soundHandler.Add("Sound\\kaboom.wav", 5);
            soundHandler.Add("Sound\\hyperspace_activate.wav", 6);

            aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width /
            (float)graphics.GraphicsDevice.Viewport.Height;
        }

        protected override void UnloadContent()
        {

        }

        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Get some input.
            UpdateInput();

            // Add velocity to the current position.
            modelPosition += modelVelocity;
            // Bleed off velocity over time.
            modelVelocity *= 0.95f;

            base.Update(gameTime);
        }

        protected void UpdateInput()
        {
            float speed = 50.0f;
            if (inputs.PauseGame)
            {
                this.Exit();
            }
            if (inputs.IsKeyPressed(Keys.Up))
            {
                soundHandler.VolumeUp(0.1f);
                modelPosition += Vector3.Up * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Down))
            {
                soundHandler.VolumeDown(0.1f);
                modelPosition += Vector3.Down * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Right))
            {
                soundHandler.Play(4);
                modelPosition += Vector3.Right * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Left))
            {
                soundHandler.Play(5);
                modelPosition += Vector3.Left * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Space))
            {
                songs[0].Play(false, 0.5f);
                modelPosition = Vector3.Zero;
                modelVelocity = Vector3.Zero;
                modelRotation = 0.0f;
                modelRotationPitch = 0.0f;
            }
            if (inputs.IsKeyDown(Keys.A))
            {
                modelRotation += 0.10f;
                //System.Diagnostics.Trace.WriteLine("modelRotation=" + modelRotation);
            }
            else if (inputs.IsKeyDown(Keys.D))
            {
                modelRotation -= 0.10f;
                //System.Diagnostics.Trace.WriteLine("modelRotation=" + modelRotation);
            }
            else if (inputs.IsKeyDown(Keys.W))
            {
                modelRotationPitch += 0.10f;
                //System.Diagnostics.Trace.WriteLine("modelRotation=" + modelRotation);
            }
            else if (inputs.IsKeyDown(Keys.S))
            {
                modelRotationPitch -= 0.10f;
                //System.Diagnostics.Trace.WriteLine("modelRotation=" + modelRotation);
            }
            else if (inputs.IsKeyPressed(Keys.L))
            {
                sounds[0].Play(false, 0.5f);
            }
            else if (inputs.IsKeyPressed(Keys.P))
            {
                sounds[1].Play(true, 0.3f);
            }
            else if (inputs.IsKeyPressed(Keys.O))
            {
                sounds[1].Stop();
            }
            modelRotation -= inputs.MouseMoved.X * 0.1f;
            modelRotationPitch -= inputs.MouseMoved.Y * 0.1f;
            
            // Get the game pad state.
            GamePadState currentState = inputs.CurrentGamePadState;
            if (currentState.IsConnected)
            {
                // Rotate the model using the left thumbstick, and scale it down.
                modelRotation -= currentState.ThumbSticks.Left.X * 0.10f;

                // Create some velocity if the right trigger is down.
                Vector3 modelVelocityAdd = Vector3.Zero;

                // Find out what direction we should be thrusting, using rotation.
                modelVelocityAdd.X = -(float)Math.Sin(modelRotation);
                modelVelocityAdd.Z = -(float)Math.Cos(modelRotation);

                // Now scale our direction by how hard the trigger is down.
                modelVelocityAdd *= currentState.Triggers.Right;

                // Finally, add this vector to our velocity.
                modelVelocity += modelVelocityAdd;

                GamePad.SetVibration(PlayerIndex.One, currentState.Triggers.Right,
                    currentState.Triggers.Right);

                // In case you get lost, press A to warp back to the center.
                if (currentState.Buttons.A == ButtonState.Pressed)
                {
                    modelPosition = Vector3.Zero;
                    modelVelocity = Vector3.Zero;
                    modelRotation = 0.0f;
                }
            }
        }

         protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // Copy any parent transforms.
            Matrix[] transforms = new Matrix[myModel.Bones.Count];
            myModel.CopyAbsoluteBoneTransformsTo(transforms);

            // Draw the model. A model can have multiple meshes, so loop.
            foreach (ModelMesh mesh in myModel.Meshes)
            {
                // This is where the mesh orientation is set, as well as our camera and projection.
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.World = transforms[mesh.ParentBone.Index] * Matrix.CreateRotationY(modelRotation) * Matrix.CreateRotationX(modelRotationPitch)
                        * Matrix.CreateTranslation(modelPosition);
                    effect.View = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
                    effect.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f),
                        aspectRatio, 1.0f, 10000.0f);
                }
                // Draw the mesh, using the effects set above.
                mesh.Draw();
            }
                base.Draw(gameTime);
        }
    }
}
